# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
"""Implements methods for serializing data for an inference endpoint."""
from __future__ import absolute_import

from typing import List, Optional

# base_serializers was refactored from serializers.
# this import ensures backward compatibility.
from sagemaker.base_serializers import (  # noqa: F401 # pylint: disable=W0611
    BaseSerializer,
    CSVSerializer,
    DataSerializer,
    IdentitySerializer,
    JSONLinesSerializer,
    JSONSerializer,
    LibSVMSerializer,
    NumpySerializer,
    SimpleBaseSerializer,
    SparseMatrixSerializer,
)

from sagemaker.jumpstart import artifacts, utils as jumpstart_utils


def retrieve_options(
    region: Optional[str] = None,
    model_id: Optional[str] = None,
    model_version: Optional[str] = None,
    tolerate_vulnerable_model: bool = False,
    tolerate_deprecated_model: bool = False,
) -> List[BaseSerializer]:
    """Retrieves the supported serializers for the model matching the given arguments.

    Args:
        region (str): The AWS Region for which to retrieve the supported serializers.
            Defaults to ``None``.
        model_id (str): The model ID of the model for which to
            retrieve the supported serializers. (Default: None).
        model_version (str): The version of the model for which to retrieve the
            supported serializers. (Default: None).
        tolerate_vulnerable_model (bool): True if vulnerable versions of model
            specifications should be tolerated (exception not raised). If False, raises an
            exception if the script used by this version of the model has dependencies with known
            security vulnerabilities. (Default: False).
        tolerate_deprecated_model (bool): True if deprecated models should be tolerated
            (exception not raised). False if these models should raise an exception.
            (Default: False).
    Returns:
        List[SimpleBaseSerializer]: The supported serializers to use for the model.

    Raises:
        ValueError: If the combination of arguments specified is not supported.
    """

    if not jumpstart_utils.is_jumpstart_model_input(model_id, model_version):
        raise ValueError(
            "Must specify JumpStart `model_id` and `model_version` when retrieving serializers."
        )

    return artifacts._retrieve_serializer_options(
        model_id,
        model_version,
        region,
        tolerate_vulnerable_model,
        tolerate_deprecated_model,
    )


def retrieve_default(
    region: Optional[str] = None,
    model_id: Optional[str] = None,
    model_version: Optional[str] = None,
    tolerate_vulnerable_model: bool = False,
    tolerate_deprecated_model: bool = False,
) -> BaseSerializer:
    """Retrieves the default serializer for the model matching the given arguments.

    Args:
        region (str): The AWS Region for which to retrieve the default serializer.
            Defaults to ``None``.
        model_id (str): The model ID of the model for which to
            retrieve the default serializer. (Default: None).
        model_version (str): The version of the model for which to retrieve the
            default serializer. (Default: None).
        tolerate_vulnerable_model (bool): True if vulnerable versions of model
            specifications should be tolerated (exception not raised). If False, raises an
            exception if the script used by this version of the model has dependencies with known
            security vulnerabilities. (Default: False).
        tolerate_deprecated_model (bool): True if deprecated models should be tolerated
            (exception not raised). False if these models should raise an exception.
            (Default: False).
    Returns:
        SimpleBaseSerializer: The default serializer to use for the model.

    Raises:
        ValueError: If the combination of arguments specified is not supported.
    """

    if not jumpstart_utils.is_jumpstart_model_input(model_id, model_version):
        raise ValueError(
            "Must specify JumpStart `model_id` and `model_version` when retrieving serializers."
        )

    return artifacts._retrieve_default_serializer(
        model_id,
        model_version,
        region,
        tolerate_vulnerable_model,
        tolerate_deprecated_model,
    )
